package resumable_connection;

import connection.*;
import ru.ifmo.is.sm.AutomatonBase;
import java.io.IOException;

public class ResumableConnection extends AutomatonBase<IConnection>
		implements IConnection {
	private ResumableConnection() {
		Socket socket = new Socket();

		//   
		IConnection connected = new ConnectedState<ResumableConnection>(this, this, socket);
		IConnection disconnected = new DisconnectedState<ResumableConnection>(this, this, socket);
		IConnection error = new ErrorState<ResumableConnection>(this, this, socket);

		//  
		addEdge(connected, ConnectedState.DISCONNECT, disconnected);
		addEdge(connected, ConnectedState.ERROR, error);
		addEdge(disconnected, DisconnectedState.CONNECT, connected);
		addEdge(disconnected, DisconnectedState.ERROR , error);
		addEdge(error, ErrorState.CONNECT, connected);
		addEdge(error, ErrorState.DISCONNECT, disconnected);

		//  
		state = disconnected;
	}

	//   
	public static IConnection createAutomaton() {
		return new ResumableConnection();
	}

	//   
	public void connect() throws IOException { state.connect(); }
	public void disconnect() throws IOException { state.disconnect(); }
	public int receive() throws IOException { return state.receive(); }
	public void send(int value) throws IOException { state.send(value); }
}
